import argparse
import os
import sys
from pathlib import Path
sys.tracebacklimit = 0
DATA_DIR = os.path.join(os.getcwd(), 'data')
PARENT_DIR = Path(os.getcwd()).parent.absolute()



def is_number(n):
    is_number = True
    try:
        num = float(n)
        # check for "nan" floats
        is_number = num == num
    except ValueError:
        is_number = False
    return is_number

class ArgumentParser() :
    def __init__(self) :
        self.parser = argparse.ArgumentParser(
                prog="axelf.py",
                formatter_class=argparse.RawDescriptionHelpFormatter,
                usage="axelf.py [-h] inputFile -s [TPM_SOURCE] [-a [ALLELE_LIST]] [-c [CANCER_TYPE]] [-l [PEPTIDE_LENGTH]] [-o [OUTPUT]]",
                description='''\
                Utilizing biophysical principles and existing MHC binding prediction tools 
                in concert with experimental RNA expression values, AXEL-F derive a function that estimates 
                the likelihood of a peptide being presented on a given MHC class I molecule.
                '''
            )

        # Positional Arguments
        self.parser.add_argument('inputFile', type=argparse.FileType('r'), nargs='+',
                            # action=SetFileExtension,
                            help='''\
                                FASTA file or CSV file containing peptides with or without additional data
                                such as alleles, gene name/id, tpm, etc.
                            ''')

        # Hidden Arguments
        self.parser.add_argument('--inputFileType', required=False, type=str,
                            nargs="?", default="", help=argparse.SUPPRESS)

        # Optional Arguments
        # TODO : Maybe add 'metavar='
        self.parser.add_argument('-s', '--tpm-source', type=str,
                            nargs="?", choices=["custom", "tcga"], default="custom", const="custom",
                            help="Specify where TPM values are coming from.")

        self.parser.add_argument('-t', '--tpm', type=float,
                            nargs="?", 
                            # action=ValidateTPM,
                            help="TPM value.")

        self.parser.add_argument('-g', '--gene-name', type=str,
                            nargs="?", 
                            # action=ValidateGeneName,
                            help="Gene name (e.g. RNF44).")

        self.parser.add_argument('-c', '--cancer-type', type=str,
                            nargs="?", 
                            # action=ValidateCancerType,
                            help="Acronym of the cancer type (e.g. GBM, LGG).")

        self.parser.add_argument('-l', '--peptide-length', type=int,
                            nargs="?", 
                            # action=ValidatePeptideLength,
                            help="Path to the file containing length of peptide.")

        self.parser.add_argument('-a', '--allele-list', type=str,
                            nargs="?", 
                            # action=ValidateAlleleList,
                            help="Path to the file containing list of alleles.")

        self.parser.add_argument('-o', '--output', type=argparse.FileType('w', encoding='UTF-8'),
                            nargs="?", help="Path to the output file.")

    def parse_arguments(self) :
        return self.parser.parse_args()